"use strict";
const StepFunctionSimulatorServer_1 = require("./StepFunctionSimulatorServer");
const StateInfoHandler_1 = require("./StateInfoHandler");
const Logger_1 = require("./utils/Logger");
const StateType_1 = require("./stateTasks/StateType");
class ServerlessOfflineStepFunctionsPlugin {
    constructor(serverless, cliOptions) {
        var _a;
        this.serverless = serverless;
        this.cliOptions = cliOptions;
        this.logger = Logger_1.Logger.getInstance();
        this.commands = {
            '@fernthedev/serverless-offline-step-functions': {
                options: {
                    port: {
                        required: false,
                        usage: 'Port of the Step Functions API Simulator (Default: 8014)',
                    },
                    enabled: {
                        required: false,
                        usage: 'Enabled Step Function API Simulator (Default: true)',
                    },
                },
            },
        };
        this.mergeOptions();
        if (((_a = this.options) === null || _a === void 0 ? void 0 : _a.enabled) === false) {
            this.logger.warning('Simulator will not execute.');
            return;
        }
        this.hooks = {
            'before:offline:start:init': this.start.bind(this),
            'offline:start:init': this.ready.bind(this),
            'offline:start:end': this.end.bind(this),
        };
    }
    start() {
        var _a, _b, _c;
        this.injectGlobalEnvVars();
        const definedStateMachines = (_b = (_a = this.serverless.service.initialServerlessConfig) === null || _a === void 0 ? void 0 : _a.stepFunctions) === null || _b === void 0 ? void 0 : _b.stateMachines;
        this.resolveHandlers(definedStateMachines);
        this.stepFunctionSimulatorServer = new StepFunctionSimulatorServer_1.StepFunctionSimulatorServer({
            port: ((_c = this.options) === null || _c === void 0 ? void 0 : _c.port) || 8014,
            stateMachines: definedStateMachines,
        });
    }
    ready() {
        var _a;
        return (_a = this.stepFunctionSimulatorServer) === null || _a === void 0 ? void 0 : _a.initServer();
    }
    async end() {
        var _a;
        await ((_a = this.stepFunctionSimulatorServer) === null || _a === void 0 ? void 0 : _a.shutdown());
    }
    mergeOptions() {
        const custom = this.serverless.service.custom;
        const customOptions = custom['@fernthedev/serverless-offline-step-functions'];
        this.options = {
            ...this.cliOptions,
            ...customOptions,
        };
    }
    getFunctionName(stateOptions) {
        var _a, _b;
        let functionName;
        const resource = stateOptions.Resource;
        if (typeof resource === 'string') {
            if (resource.endsWith('.waitForTaskToken')) {
                functionName = (_b = (_a = stateOptions.Parameters) === null || _a === void 0 ? void 0 : _a.FunctionName) === null || _b === void 0 ? void 0 : _b['Fn::GetAtt'][0];
            }
            else {
                functionName = resource.split('-').slice(-1)[0];
            }
        }
        else {
            for (const [key, value] of Object.entries(resource)) {
                if (key === 'Fn::GetAtt') {
                    functionName = value[0];
                }
            }
        }
        if (!functionName) {
            throw Error(`Could not find funciton name for resource ${resource}`);
        }
        return functionName;
    }
    setStateInfo(states, stateMachineName) {
        var _a, _b;
        const definedFunctions = this.serverless.service.initialServerlessConfig.functions;
        const statesInfoHandler = StateInfoHandler_1.StateInfoHandler.getInstance();
        this.logger.debug(`ServerlessOfflineStepFunctionsPlugin - setStateInfo - ${states}`);
        for (const [stateName, stateOptions] of states) {
            if (stateOptions.Type === StateType_1.StateType.Map) {
                const stateDefinition = Object.entries(stateOptions.Iterator.States);
                this.setStateInfo(stateDefinition, stateMachineName);
                continue;
            }
            if (stateOptions.Type !== StateType_1.StateType.Task) {
                continue;
            }
            const functionName = this.getFunctionName(stateOptions);
            const { handler } = definedFunctions[functionName];
            const indexOfHandlerNameSeparator = handler.lastIndexOf('.');
            const handlerPath = handler.substring(0, indexOfHandlerNameSeparator);
            const handlerName = handler.substring(indexOfHandlerNameSeparator + 1);
            const environment = (_b = (_a = this.serverless.service.initialServerlessConfig) === null || _a === void 0 ? void 0 : _a.functions[functionName]) === null || _b === void 0 ? void 0 : _b.environment;
            statesInfoHandler.setStateInfo(stateMachineName, stateName, handlerPath, handlerName, environment);
        }
    }
    resolveHandlers(definedStateMachines) {
        const definedStateMachinesArr = Object.entries(definedStateMachines);
        for (const [stateMachineName, stateMachineOptions] of definedStateMachinesArr) {
            const states = Object.entries(stateMachineOptions.definition.States);
            this.setStateInfo(states, stateMachineName);
        }
    }
    injectGlobalEnvVars() {
        var _a, _b;
        const providerEnvironment = (_b = (_a = this.serverless.service.initialServerlessConfig) === null || _a === void 0 ? void 0 : _a.provider) === null || _b === void 0 ? void 0 : _b.environment;
        if (providerEnvironment) {
            Object.entries(providerEnvironment).forEach(([key, value]) => {
                process.env[key] = value;
            });
        }
    }
}
module.exports = ServerlessOfflineStepFunctionsPlugin;
