"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Retrier = void 0;
const Logger_1 = require("../utils/Logger");
class Retrier {
    constructor(_ErrorEquals, _IntervalSeconds = 1, _MaxAttempts = 3, _BackoffRate = 2.0) {
        this._ErrorEquals = _ErrorEquals;
        this._IntervalSeconds = _IntervalSeconds;
        this._MaxAttempts = _MaxAttempts;
        this._BackoffRate = _BackoffRate;
        this.currentNumberOfRetries = 0;
        this.currentIntervalSeconds = _IntervalSeconds;
        this.logger = Logger_1.Logger.getInstance();
    }
    static create(taskRetryRule) {
        return new Retrier(taskRetryRule.ErrorEquals, taskRetryRule.IntervalSeconds, taskRetryRule.MaxAttempts, taskRetryRule.BackoffRate);
    }
    async retry(fn, context) {
        let output;
        try {
            output = await fn();
            return output;
        }
        catch (error) {
            this.logger.error(`Caught an error in Retrier for ${context.StateMachine.Name}-${context.State.Name}: ${error.stack}`);
            if (this.currentNumberOfRetries < this._MaxAttempts) {
                this.currentNumberOfRetries++;
                this.logger.log(`Retrying ${context.StateMachine.Name}-${context.State.Name}, attempt #${this.currentNumberOfRetries}`);
                return await new Promise((resolve, reject) => {
                    setTimeout(async () => {
                        this.currentIntervalSeconds = this.currentIntervalSeconds * this._BackoffRate;
                        try {
                            const output = await this.retry(fn, context);
                            return resolve(output);
                        }
                        catch (error) {
                            return reject(error);
                        }
                    }, this.currentIntervalSeconds * 1000);
                });
            }
            else {
                this.logger.log(`Already retried MaxAttemps of ${this.currentNumberOfRetries} for ${context.StateMachine.Name}-${context.State.Name}`);
                throw error;
            }
        }
    }
    get ErrorEquals() {
        return this._ErrorEquals;
    }
}
exports.Retrier = Retrier;
